/*
 * auditd-plugin-clickhouse-lite is a lightweight plugin for auditd
 * daemon to send audit data to a Clickhouse database.
 *
 * Copyright (C) 2023  BaseALT /basealt.ru/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "client.hpp"
#include <string>
#include <clickhouse-cpp/client.h>

class ClickhouseClient: public ClientImpl {
public:
    ClickhouseClient(const clickhouse::ClientOptions& opts);
    ~ClickhouseClient() = default;
    virtual void Execute(const clickhouse::Query& query) override;
    virtual void Select(const std::string& query,
                        clickhouse::SelectCallback cb) override;
    virtual void Insert(const std::string& table_name,
                        const clickhouse::Block& block) override;

protected:
    std::shared_ptr<clickhouse::Client> m_client;
};

ClickhouseClient::ClickhouseClient(const clickhouse::ClientOptions& opts) {
    m_client = std::make_shared<clickhouse::Client>(opts);
}

void ClickhouseClient::Execute(const clickhouse::Query& query) {
    m_client->Execute(query);
}

void ClickhouseClient::Select(const std::string& query, clickhouse::SelectCallback cb) {
    m_client->Select(query, cb);
}

void ClickhouseClient::Insert(const std::string& table_name, const clickhouse::Block& block) {
    m_client->Insert(table_name, block);
}

Client::Client(const clickhouse::ClientOptions& opts) {
    m_client = std::make_shared<ClickhouseClient>(opts);
}
