/*
 * auditd-plugin-clickhouse-lite is a lightweight plugin for auditd
 * daemon to send audit data to a Clickhouse database.
 *
 * Copyright (C) 2023  BaseALT /basealt.ru/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include <string>
#include <clickhouse-cpp/client.h>

/* These numbers are queried from a test ClickHouse
database using errorCodeToName() function. */
#define CANNOT_ALLOCATE_MEMORY 173
#define MEMORY_LIMIT_EXCEEDED  241
#define LIMIT_EXCEEDED         290

class ClientImpl {
public:
    virtual void Execute(const clickhouse::Query& query) = 0;
    virtual void Select(const std::string& query,
                        clickhouse::SelectCallback cb) = 0;
    virtual void Insert(const std::string& table_name,
                        const clickhouse::Block& block) = 0;

protected:
    ClientImpl() = default;
    ~ClientImpl() = default;
};

class Client {
public:
    Client(const clickhouse::ClientOptions& opts);
    ~Client() = default;

    void Execute(const clickhouse::Query& query) {
        m_client->Execute(query);
    };
    void Select(const std::string& query, clickhouse::SelectCallback cb) {
        m_client->Select(query, cb);
    }
    void Insert(const std::string& table_name,
                const clickhouse::Block& block) {
        m_client->Insert(table_name, block);
    };

protected:
    std::shared_ptr<ClientImpl> m_client;
};
