/*
 * auditd-plugin-clickhouse-lite is a lightweight plugin for auditd
 * daemon to send audit data to a Clickhouse database.
 *
 * Copyright (C) 2023  BaseALT /basealt.ru/
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#ifndef AUDITD_PLUGIN_CLICKHOUSE_LOGGING_HPP
#define AUDITD_PLUGIN_CLICKHOUSE_LOGGING_HPP

#include <string>
#include <list>
#include <mutex>
#include <utility>
#include <memory>

#include <stdio.h>
#include <stdarg.h>

class LoggerImpl
{
public:
    virtual void log(std::string timestamp, std::string message) = 0;
    virtual void log(std::string message) = 0;
    ~LoggerImpl() = default;

protected:
    LoggerImpl() = default;
};

class Logger
{
public:
    Logger() = default;
    ~Logger();

    static bool has_instance();
    static void write(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
    static void set_instance(std::shared_ptr<LoggerImpl> logger);
    static void set_default_instance(std::shared_ptr<LoggerImpl> logger);
    static std::string getdatestring();

private:
    static void dump();

    static std::list<std::pair<std::string, std::string> > s_lines;
    static std::recursive_mutex s_mutex;
    static std::shared_ptr<LoggerImpl> s_logger;
};

class FileLogger: public LoggerImpl
{
public:
    FileLogger(const char *filename);
    ~FileLogger() = default;

protected:
    virtual void log(std::string timestamp, std::string message) override;
    virtual void log(std::string message) override;
    std::string m_logfile_name;
};

class SyslogLogger: public LoggerImpl
{
public:
    SyslogLogger(const char *ident);
    ~SyslogLogger();

protected:
    virtual void log(std::string timestamp, std::string message) override;
    virtual void log(std::string message) override;
};

#endif /* AUDITD_PLUGIN_CLICKHOUSE_LOGGING_HPP */
